from __future__ import annotations

try:
    from importlib.metadata import PackageNotFoundError, version
except Exception:  # pragma: no cover
    PackageNotFoundError = Exception  # type: ignore[assignment]
    version = None  # type: ignore[assignment]


__dist_name__ = "prasad-lang"

try:
    __version__ = version(__dist_name__) if version else "0.0.0"
except PackageNotFoundError:
    __version__ = "0.0.0"
